#include <msp430.h>
#include <stdint.h>

#define __nop __no_operation
#define LED 0x20                            // P1.5

#pragma NOINIT ( RAM_XE )
static uint16_t RAM_XE[6];

static void dma_rearm(void)
{
    // Repeated block transfer, increase src address, increase dst address
    DMA2CTL = (uint16_t)DMADT_5 | (uint16_t)DMASRCINCR | (uint16_t)DMADSTINCR;

    // these are loaded when next DMA2CTL |= DMAEN is executed
    DMA2SA = (uint16_t)(uintptr_t)RAM_XE;
    // DMA2SA=RAM_XE[0/1/2] -> DMA2DA=SAPH_APGC, SAPH_APGLPER, SAPH_APGHPER are setup for consumption here (3 words per block)
    DMA2DA = (uint16_t)(uintptr_t)&SAPHPGC;
    // enable DMA2: this will load above DMA2SA and DMA2DA
    DMA2CTL |= (uint16_t)DMAEN;
    // these are loaded when prev. DMA2 transfer would end
    DMA2SA = (uint16_t)(uintptr_t)(RAM_XE + 3U);
    // pending next DMA2SA=RAM_XE[3/4/5] -> DMA2DA=SAPH_AXPGCTL, SAPH_AXPGLPER, SAPH_AXPGHPER are setup for consumption here (3 words per block)
    DMA2DA = (uint16_t)(uintptr_t)&SAPHXPGCTL;
}


void main(void) {
    WDTCTL = WDTPW | WDTHOLD;               // Stop watchdog timer
    PM5CTL0 &= ~LOCKLPM5;                   // Disable the GPIO power-on default high-impedance mode
                                            // to activate previously configured port settings
    P1DIR |= LED;                           // Set P1.0 to output direction

    for(;;) {
        volatile unsigned int i;            // volatile to prevent optimization

        P1OUT ^= LED;                       // Toggle P1.0 using exclusive-OR

        dma_rearm();

        i = 10000;                          // SW Delay
        do i--;
        while(i != 0);
    }
}
